#include <avr/io.h>
#include "debug.h"
#include <avr/interrupt.h>
#include "simpleio.h"
#include "m328p.h"
#include "prozeda.h"
#include "common.h"

#define SPI_MISO IOPIN(B, 4)

prozeda_logdata_t proz_logdata;

ISR(SPI_STC_vect)
{
	prozeda_rxData(SPDR);
}

ISR(TIMER0_COMPA_vect)
{
	prozeda_tick();
}

inline void print_logdata(void)
{
	char buff[10];
	
	uart_debug_writeln("%2u.%2u. %2u:%2u:%2u",
		proz_logdata.timestamp_date % 100, proz_logdata.timestamp_date / 100,
		proz_logdata.timestamp_time / 60, proz_logdata.timestamp_time % 60, proz_logdata.timestamp_seconds
	);
	
	for(uint8_t i = 0; i < 11; i++)
	{
		itoa_fixed(proz_logdata.temperature[i], buff, 1);
		uart_debug_writeln(" T[%u]: %s C", i, buff);
	}
	
	for(uint8_t i = 0; i < 8; i++)
	{
		uart_debug_writeln(" out[%u]: %u", i, proz_logdata.output[i]);
	}
	
	for(uint8_t i = 0; i < 4; i++)
	{
		uart_debug_writeln(" error[%u]: %u", i, proz_logdata.error[i]);
	}
	
	
	uart_debug_writeln(" storage[0]: %u", proz_logdata.storage0);
	uart_debug_writeln(" storage[1]: %u", proz_logdata.storage1);
	
	
	for(uint8_t i = 0; i < 4; i++)
	{
		uart_debug_writeln(" func[%u]: %u", i, proz_logdata.functionActive[i]);
	}
	
	itoa_fixed(proz_logdata.flowRate, buff, 1);
	uart_debug_writeln(" flowRate: %s", buff);
	
	itoa_fixed(proz_logdata.tapping, buff, 1);
	uart_debug_writeln(" tapping: %s", buff);
}

void print_logdata_json(void)
{
	char buff[10];
	
	uart_debug_write("{ \"time\" : \"%2u.%2u. %2u:%2u:%2u\", ",
	proz_logdata.timestamp_date % 100, proz_logdata.timestamp_date / 100,
	proz_logdata.timestamp_time / 60, proz_logdata.timestamp_time % 60, proz_logdata.timestamp_seconds
	);
	
	uart_debug_write("\"temps\" : [");
	
	for(uint8_t i = 0; i < 11; i++)
	{
		itoa_fixed(proz_logdata.temperature[i], buff, 1);
		uart_debug_write(" %s", buff);
		if(i < 10)
		{
			uart_debug_putc(',');
		}
	}
	
	uart_debug_write(" ], \"out\" : [");
	for(uint8_t i = 0; i < 8; i++)
	{
		uart_debug_write(" %u", proz_logdata.output[i]);
		if(i < 7)
		{
			uart_debug_putc(',');
		}
	}
	
	uart_debug_write(" ], \"error\" : [");
	for(uint8_t i = 0; i < 4; i++)
	{
		uart_debug_write(" %u", proz_logdata.functionActive[i]);
		if(i < 3)
		{
			uart_debug_putc(',');
		}
	}
	
	uart_debug_write(" ], \"storage\" : [ %u, %u], \"func\" : [", proz_logdata.storage0, proz_logdata.storage1);
	for(uint8_t i = 0; i < 4; i++)
	{
		uart_debug_write(" %u", proz_logdata.functionActive[i]);
		if(i < 3)
		{
			uart_debug_putc(',');
		}
	}
	
	itoa_fixed(proz_logdata.flowRate, buff, 1);
	uart_debug_write("], \"flowRate\" : %s", buff);
	itoa_fixed(proz_logdata.tapping, buff, 1);
	uart_debug_writeln(", \"tapping\" : %s }", buff);
	
}

int main(void)
{
	uart_debug_init();
	
	IOPIN_O(SPI_MISO);
	// enable SPI slave
	SPCR = (1<<SPIE) | (1<<SPE) | (0<<DORD) | (0<<MSTR) | (0<<CPOL) | (0<<CPHA);
	
	// setup timer
	TCCR0A = timer0a_wgm_ctc;
	TCCR0B = timer0b_wgm_ctc | timer0_prescaler_8;
	OCR0A = 150; // ca. 100 s @ 12 MHz
	TIMSK0 = 1<<OCIE0A;
	
	//init debug pins
	//IOPIN_O(DBG0);
	//IOPIN_L(DBG0);
	//IOPIN_O(DBG1);
	//IOPIN_L(DBG1);
	//IOPIN_O(DBG2);
	//IOPIN_L(DBG2);
	
	prozeda_logdata_ptr = (uint8_t *)&proz_logdata;
		
	sei();
	
	uart_debug_writeln("Built on: " __DATE__ " " __TIME__);
	
    while (1) 
    {
		prozeda_taskstatus_t status = prozeda_task();
		
		if(status == prozeda_taskstatus_rxLogdata)
		{
			print_logdata_json();
		}
		else if(status == prozeda_taskstatus_rxError)
		{
			uart_debug_writeln("rxErr");
		}
    }
}

