#include <avr/pgmspace.h>
#include "prozeda.h"
#include "debug.h"

#ifdef PROZEDA_SUPPORT_SYSINFO
	volatile uint16_t prozeda_bytepos = 0;
#else
	volatile uint8_t prozeda_bytepos = 0;
#endif

prozeda_msgmode_t prozeda_msgmode = prozeda_msgmode_unknown;
volatile uint8_t prozeda_ticks = 0;
volatile uint8_t prozeda_buff[PROZEDA_BUFF_LEN];
volatile bool prozeda_process = false;

uint8_t* prozeda_logdata_ptr = NULL;

bool prozeda_checkMessage(uint8_t length, uint8_t expLength)
{
	//messages with length less 2 bytes are invalid
	if(expLength < 2 || expLength != length)
	{
		return false;
	}
	
	uint8_t sum = 0;
	expLength--;
	uint8_t checksum = prozeda_buff[expLength];
	while(expLength--)
	{
		sum += prozeda_buff[expLength];
	}
	
	if(sum == checksum)
	{
		return true;
	}
	return false;
}

prozeda_taskstatus_t prozeda_task(void)
{
	if(prozeda_process == false)
	{
		return prozeda_taskstatus_none;
	}
	//IOPIN_H(DBG2);
	prozeda_process = false;
	
	uint8_t len = prozeda_bytepos;
	prozeda_bytepos = 0;
	prozeda_spi_disable();
	prozeda_spi_enable();
	
	prozeda_taskstatus_t status = prozeda_taskstatus_none;
	
	if(len == PROZEDA_MSGLEN_MSGTYPE)
	{
		//everything ok, now parse the data
		//check if it's a message type switcher
		if(arrcmp((const char*)prozeda_buff, "\xAA\x55\x55\xAA", 4))
		{
			prozeda_msgmode = (prozeda_buff[4] << 8) | prozeda_buff[5];
			status = prozeda_taskstatus_rxMsgtype;
		}
	}
	else if(prozeda_msgmode == prozeda_msgmode_logdata)
	{
		if(prozeda_checkMessage(len, PROZEDA_MSGLEN_LOGDATA))
		{
			if(prozeda_logdata_ptr != NULL)
			{
				memcpy(prozeda_logdata_ptr, (const void*)prozeda_buff, PROZEDA_MSGLEN_LOGDATA);
				status = prozeda_taskstatus_rxLogdata;
			}
			// return another status if ptr is NULL?
		}
		else
		{
			status = prozeda_taskstatus_rxError;
		}
	}
	else
	{
		status = prozeda_taskstatus_rxNotImplemented;
	}
	//IOPIN_L(DBG2);
	return status;
}